//-----------------------------------------------------------------------
#include <iostream>

#include "INetworkGame.h"
#include "IGhost.h"
#include "IGhostMaster.h"
#include "CException.h"
//-----------------------------------------------------------------------
#include <iostream>
//-----------------------------------------------------------------------
INetworkGame::INetworkGame() : INetworkUser()
{
	m_iNextId=0;
	m_iLastId=-1;
	m_bUpdateGhostEnabled=false;
	m_bIdsReserved=false;
}
//-----------------------------------------------------------------------
INetworkGame::~INetworkGame()
{
	for (ghost_iterator I=m_Ghosts.begin();I!=m_Ghosts.end();I++)
		delete ((*I).second);
	m_Ghosts.clear();
	m_GhostMasters.clear();
}
//-----------------------------------------------------------------------
void INetworkGame::onNetworkReady()
{
	// cerr << "onNetworkReady" << endl;
	if (!m_Ntwrk->isServer())
	{
		requestIds();
		requestGhosts();
	}
	else
	{
		m_bUpdateGhostEnabled=true;
		m_bIdsReserved=true;  
		//cerr << "game server ok." << endl;
	}
}
//-----------------------------------------------------------------------
int  INetworkGame::nextGhostId()
{
	if (!m_Ntwrk->isServer())
	{
		if (m_iNextId == m_iLastId)
			throw CException("INetworkGame: No more GhostMaster ids !!");
	}
	return (m_iNextId++);
}
//-----------------------------------------------------------------------
void INetworkGame::prepareUpdateGhostData(IGhostMaster *gm,
										  unsigned char*& _data,
										  int& _l)
{
	ng_update_ghost       g;
	int                   l;
	static unsigned char *data=NULL;
	static int            ldata=0;

	// preparing ghost infos
	g.m_iId=gm->getGhostMasterId();
	// -> getting master data
	l=gm->getGhostMasterDataSize()+sizeof(ng_update_ghost);
	if (l > ldata)
	{
		if (data != NULL)
			delete [](data);
		data=new unsigned char[l];
		ldata=l;
	}
	memcpy(data,&g,sizeof(g));
	memcpy(data+sizeof(ng_update_ghost),
		gm->getGhostMasterData(),gm->getGhostMasterDataSize());
	_data=data;
	_l=l;
}
//-----------------------------------------------------------------------
void INetworkGame::prepareNewGhostData(IGhostMaster *gm,
									   unsigned char*& _data,
									   int& _l)
{
	ng_new_ghost    g;
	int             l;
	static unsigned char *data=NULL;
	static int            ldata=0;

	// preparing ghost infos
	g.m_iId=gm->getGhostMasterId();
	g.m_iType=gm->getGhostMasterType();
	g.m_iIdMaster=m_Ntwrk->clientId();
	// -> getting master data
	l=gm->getGhostMasterInitDataSize()+sizeof(ng_new_ghost);
	if (l > ldata)
	{
		if (data != NULL)
			delete [](data);
		data=new unsigned char[l];
		ldata=l;
	}
	memcpy(data,&g,sizeof(ng_new_ghost));
	memcpy(data+sizeof(ng_new_ghost),
		gm->getGhostMasterInitData(),gm->getGhostMasterInitDataSize());
	_data=data;
	_l=l;
}
//-----------------------------------------------------------------------
void INetworkGame::prepareNewGhostData(IGhost *gh,
									   unsigned char*& _data,
									   int& _l)
{
	ng_new_ghost          g;
	int                   l;
	static unsigned char *data=NULL;
	static int            ldata=0;

	// preparing ghost infos
	g.m_iId=gh->getGhostId();
	g.m_iType=gh->getGhostType();
	g.m_iIdMaster=gh->getMasterClientId();
	// -> getting ghost data
	l=gh->getGhostInitDataSize()+sizeof(ng_new_ghost);
	if (l > ldata)
	{
		if (data != NULL)
			delete [](data);
		data=new unsigned char[l];
		ldata=l;
	}
	memcpy(data,&g,sizeof(g));
	memcpy(data+sizeof(ng_new_ghost),
		gh->getGhostInitData(),gh->getGhostInitDataSize());
	_data=data;
	_l=l;
}
//-----------------------------------------------------------------------
void INetworkGame::registerGhostMaster(IGhostMaster *gm)
{
	ghostmaster_iterator G;
	network_msg          m;
	int                  id,l=0;
	unsigned char       *data=NULL;

  /*
	if (!isNetworkRunning())
		return;
  */
	if (m_bIdsReserved || !isNetworkRunning())
	{
		id=nextGhostId();
		// adding ghost master
		G=m_GhostMasters.find(id);
		if (G != m_GhostMasters.end())
			throw CException("INetworkGame: Duplicate ghost master ! (id=%d)",id);
		m_GhostMasters[id]=gm;
		gm->setGhostMasterId(id);
		gm->setNetworkGame(this);
  	if (isNetworkRunning())
    {
		  // preparing data
		  prepareNewGhostData(gm,data,l);
		  // sending
		  m_Ntwrk->clientMessageFromData(NG_MSG_NEW_GHOST,data,l,&m);
		  m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_OTHERS,&m);
    }
		//cerr << "GhostMaster " << id << " registred." << endl;
	}
	/*
	else
		cerr << "A ghost registration was canceled ..." << endl;
	*/
}
//-----------------------------------------------------------------------
void INetworkGame::unregisterGhostMaster(IGhostMaster *gm)
{
	network_msg          m;
	ng_kill_ghost        g;

	// preparing kill ghost infos
	g.m_iId=gm->getGhostMasterId();
	// sending
	m_Ntwrk->clientMessageFromData(NG_MSG_KILL_GHOST,(unsigned char *)&g,sizeof(ng_kill_ghost),&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_OTHERS,&m);
	// remove from ghost master list
	m_GhostMasters.erase(gm->getGhostMasterId());
	//cerr << "GhostMaster " << gm->getGhostMasterId() << " unregistred." << endl;
}
//-----------------------------------------------------------------------
void INetworkGame::removeGhost(IGhost *g)
{
	m_Ghosts.erase(g->getGhostId());
}
//-----------------------------------------------------------------------
void INetworkGame::updateGhostsOf(IGhostMaster *gm)
{
	network_msg       m;
	int               l=0;
	unsigned char    *data=NULL;

	// preparing data
	prepareUpdateGhostData(gm,data,l);
	// sending
	m_Ntwrk->clientMessageFromData(NG_MSG_UPDATE_GHOST,data,l,&m);
	m_Ntwrk->clientSend(gm->getUpdateMode(),NTWRK_TO_OTHERS,&m);
}
//-----------------------------------------------------------------------
void INetworkGame::requestIds()
{
	network_msg m;
	//cerr << "requestIds" << endl;
	int         n=NG_NB_IDS_REQUESTED;
	m_Ntwrk->clientMessageFromData(NG_MSG_REQUEST_IDS,
		(unsigned char *)&n,sizeof(n),&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_SERVER,&m);
}
//-----------------------------------------------------------------------
void INetworkGame::requestGhosts()
{
	network_msg m;
	//cerr << "requestGhosts" << endl;
	m_Ntwrk->clientMessageFromData(NG_MSG_REQUEST_GHOSTS,NULL,0,&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_SERVER,&m);  
}
//-----------------------------------------------------------------------
void INetworkGame::waitForNetworkReady()
{
	INetworkUser::waitForNetworkReady();
	int startTime=SDL_GetTicks();
	//m_Ntwrk->print("Waiting for ids ...\n");
	while (SDL_GetTicks() - startTime < NU_TMOUT)
	{
		m_Ntwrk->processMessages();
		if (m_bIdsReserved)
			return;
		SDL_Delay(NU_TMOUT/10);
	}
	throw CException("INetworkGame::waitForNetworkReady - no answer from network");
}
//-----------------------------------------------------------------------
void INetworkGame::processServerMessage(const network_msg& m)
{
	network_msg r;

	switch (m.m_Type)
	{
	case NG_MSG_REQUEST_IDS:
		{
			ng_ids ids;
			int   *n;

			//cerr << "NG_MSG_REQUEST_IDS from " << (int)m.m_ClientId << endl;
			n=reinterpret_cast<int *>(m.m_Data.m_Raw);
			ids.m_iFirst=m_iNextId;
			ids.m_iLast=m_iNextId+*n;
			m_iNextId+=*n + 1;
			//cerr << "  reserving " << *n << " ids nextId=" << m_iNextId << endl;
			m_Ntwrk->serverMessageFromData(NG_MSG_IDS,
				(unsigned char *)&ids,
				sizeof(ids),&r);
			m_Ntwrk->serverSend(NTWRK_SECURE,m.m_ClientId,&r);
			break;
		}
	case NG_MSG_REQUEST_GHOSTS:
		{
			//cerr << "NG_MSG_REQUEST_GHOSTS from " << (int)m.m_ClientId << endl;
			sendGhosts(m.m_ClientId);
			break;
		}
	}
}
//-----------------------------------------------------------------------
void INetworkGame::processClientMessage(const network_msg& m)
{
	switch (m.m_Type)
	{
	case NG_MSG_NEW_GHOST:
		{
			IGhost        *ng;
			ng_new_ghost  *g;
			unsigned char *data;

			g=reinterpret_cast<ng_new_ghost *>(m.m_Data.m_Raw);
			//cerr << "NG_MSG_NEW_GHOST from " << (int)m.m_ClientId << " ghost id = " << g->m_iId << endl;
			data=m.m_Data.m_Raw+sizeof(ng_new_ghost);
			ng=createGhost(g->m_iType,g->m_iId,
				data,m.m_Data.m_iSize-sizeof(ng_new_ghost));
			if (ng != NULL)
			{
				ghost_iterator    G=m_Ghosts.find(g->m_iId);
				if (G == m_Ghosts.end())
				{
					m_Ghosts[g->m_iId]=ng;
					ng->setNetworkGame(this);
					ng->setMasterClientId(g->m_iIdMaster);
					// events
					ng->onGhostReady();
					onNewGhost(ng);
				}
				/*
				else
					throw CException("INetworkGame: Duplicate Ghost ! (id=%d)",g->m_iId);
					cerr << "INetworkGame: Duplicate new Ghost ! (" << g->m_iId << ")" << endl;
				*/
			}
			else
				throw CException("INetworkGame: Ghost creation failed ! (id=%d type=%d)",
				g->m_iId,g->m_iType);
			break;
		}
	case NG_MSG_KILL_GHOST:
		{
			ghost_iterator G;

			//cerr << "NG_MSG_KILL_GHOST from " << (int)m.m_ClientId << endl;
			if (m_bUpdateGhostEnabled)
			{
				ng_kill_ghost *g=reinterpret_cast<ng_kill_ghost *>(m.m_Data.m_Raw);
				//cerr << "  Ghost id = " << g->m_iId << endl;
				G=m_Ghosts.find(g->m_iId);
				if (G != m_Ghosts.end())
				{
					(*G).second->onGhostMasterDied();
					m_Ghosts.erase(g->m_iId);
				}
				else
					cerr << "INetworkGame::NG_MSG_KILL_GHOST Cannot found ghost (id=" << g->m_iId << ")" << endl;
			}
			break;
		}
	case NG_MSG_IDS:
		{
			ng_ids *ids;
			//cerr << "NG_MSG_IDS from " << (int)m.m_ClientId << endl;
			ids=reinterpret_cast<ng_ids *>(m.m_Data.m_Raw);
			m_iNextId=ids->m_iFirst;
			m_iLastId=ids->m_iLast;
			//cerr << "  reserved ids are [" << m_iNextId 
			//	<< ',' << m_iLastId << ']' << endl;
			m_bIdsReserved=true;
			break;
		}
	case NG_MSG_UPDATE_GHOST:
		{
			ghost_iterator    G;
			ng_update_ghost  *g;
			unsigned char    *data;
			int               l;

			//cerr << "NG_MSG_UPDATE_GHOST from " << (int)m.m_ClientId << endl;
			if (m_bUpdateGhostEnabled)
			{
				g=reinterpret_cast<ng_update_ghost *>(m.m_Data.m_Raw);
				data=m.m_Data.m_Raw+sizeof(ng_update_ghost);
				l=m.m_Data.m_iSize-sizeof(ng_update_ghost);
				G=m_Ghosts.find(g->m_iId);
				if (G != m_Ghosts.end())
					(*G).second->setGhostData(data,l);
				else
					cerr << "INetworkGame::NG_MSG_UPDATE_GHOST Cannot found ghost (id=" << g->m_iId << ")" << endl;
			}
			break;
		}
	case NG_MSG_GHOST:
		{
			ghostmaster_iterator    G;
			ng_ghost_msg           *g;
			unsigned char          *data;
			int                     l;

			//cerr << "NG_MSG_GHOST from " << (int)m.m_ClientId << endl;
			if (m_bUpdateGhostEnabled)
			{
				g=reinterpret_cast<ng_ghost_msg *>(m.m_Data.m_Raw);
				data=m.m_Data.m_Raw+sizeof(ng_ghost_msg);
				l=m.m_Data.m_iSize-sizeof(ng_ghost_msg);
				G=m_GhostMasters.find(g->m_iId);
				if (G != m_GhostMasters.end())
					(*G).second->messageFromGhost(m.m_ClientId,g,data,l);
				else
					throw CException("INetworkGame::NG_MSG_GHOST Cannot found ghost master (id=%d)",
					g->m_iId);
			}
			break;
		}
	case NG_MSG_MASTER:
		{
			ghost_iterator          G;
			ng_ghost_msg           *g;
			unsigned char          *data;
			int                     l;

			if (m_bUpdateGhostEnabled)
			{
				//cerr << "NG_MSG_MASTER from " << (int)m.m_ClientId << endl;
				g=reinterpret_cast<ng_ghost_msg *>(m.m_Data.m_Raw);
				data=m.m_Data.m_Raw+sizeof(ng_ghost_msg);
				l=m.m_Data.m_iSize-sizeof(ng_ghost_msg);
				G=m_Ghosts.find(g->m_iId);
				if (G != m_Ghosts.end())
					(*G).second->messageFromMaster(g,data,l);
				else
					throw CException("INetworkGame::NG_MSG_MASTER Cannot found ghost (id=%d)",
					g->m_iId);
			}
			break;
		}
	case NG_MSG_END_OF_GHOSTS:
		{
			network_msg msg;

			//cerr << "NG_MSG_END_OF_GHOSTS from " << (int)m.m_ClientId << endl;
			// all ghosts are known - accept updates
			m_bUpdateGhostEnabled=true;
			// warn everybody
			m_Ntwrk->clientMessageFromData(NG_MSG_ALL_GHOSTS_RECEIVED,NULL,0,&msg);
			m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_OTHERS,&msg);
			break;
		}
	case NG_MSG_ALL_GHOSTS_RECEIVED:
		{
			//cerr << "NG_MSG_ALL_GHOSTS_RECEIVED from " << (int)m.m_ClientId << endl;
			onAllGhostsReceived((int)m.m_ClientId);
			break;
		}
	}
}
//-----------------------------------------------------------------------
void INetworkGame::sendGhosts(Uint8 cid)
{
	network_msg    m;
	unsigned char *data=NULL;
	int            l=0;

	for (ghostmaster_iterator M=m_GhostMasters.begin();
		M != m_GhostMasters.end();M++)
	{
		// preparing data
		prepareNewGhostData((*M).second,data,l);
		// sending
		m_Ntwrk->serverMessageFromData(NG_MSG_NEW_GHOST,data,l,&m);
		m_Ntwrk->serverSend(NTWRK_SECURE,cid,&m);
	}
	for (ghost_iterator I=m_Ghosts.begin();
		I != m_Ghosts.end();I++)
	{
		// preparing data
		prepareNewGhostData((*I).second,data,l);
		// sending
		m_Ntwrk->serverMessageFromData(NG_MSG_NEW_GHOST,data,l,&m);
		m_Ntwrk->serverSend(NTWRK_SECURE,cid,&m);
	}
	// end of ghost
	m_Ntwrk->serverMessageFromData(NG_MSG_END_OF_GHOSTS,NULL,0,&m);
	m_Ntwrk->serverSend(NTWRK_SECURE,cid,&m);
}
//-----------------------------------------------------------------------
void INetworkGame::processGhosts()
{
	if (!isNetworkRunning())
    return;

	for (ghostmaster_iterator I=m_GhostMasters.begin();
		I != m_GhostMasters.end();I++)
	{
		if ((*I).second->needUpdate())
		{
			updateGhostsOf((*I).second);
			(*I).second->setNeedUpdate(false);
		}
	}  
}
//-----------------------------------------------------------------------
void INetworkGame::prepareGhostMsgData(int id,int info,
									   const unsigned char *din,
									   int                  lin,
									   unsigned char*&      dout,
									   int&                 lout)
{
	ng_ghost_msg          g;
	int                   l;
	static unsigned char *sData=NULL;
	static int            sLData=0;

	// preparing ghost infos
	g.m_iId=id;
	g.m_iInfo=info;
	l=lin+sizeof(ng_ghost_msg);
	// -> getting master data
	if (l > sLData)
	{
		if (sData != NULL)
			delete [](sData);
		sData=new unsigned char[l];
		sLData=l;
	}
	memcpy(sData,&g,sizeof(g));
	memcpy(sData+sizeof(g),din,lin);
	dout=sData;
	lout=sLData;
}
//-----------------------------------------------------------------------
void INetworkGame::ghostToMaster(IGhost *g,int t,
								 const unsigned char *d,int l)
{
	unsigned char *dm;
	int            lm;
	network_msg    m;

	prepareGhostMsgData(g->getGhostId(),t,d,l,dm,lm);
	m_Ntwrk->clientMessageFromData(NG_MSG_GHOST,dm,lm,&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,g->getMasterClientId(),&m);
}
//-----------------------------------------------------------------------
void INetworkGame::masterToGhost(IGhostMaster *g,int w,int t,
								 const unsigned char *d,int l)
{
	unsigned char *dm;
	int            lm;
	network_msg    m;

	prepareGhostMsgData(g->getGhostMasterId(),t,d,l,dm,lm);
	m_Ntwrk->clientMessageFromData(NG_MSG_MASTER,dm,lm,&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,w,&m);
}
//-----------------------------------------------------------------------
void INetworkGame::masterToGhosts(IGhostMaster *g,int t,
								  const unsigned char *d,int l)
{
	unsigned char *dm;
	int            lm;
	network_msg    m;

	prepareGhostMsgData(g->getGhostMasterId(),t,d,l,dm,lm);
	m_Ntwrk->clientMessageFromData(NG_MSG_MASTER,dm,lm,&m);
	m_Ntwrk->clientSend(NTWRK_SECURE,NTWRK_TO_OTHERS,&m);
}
//-----------------------------------------------------------------------
IGhost *INetworkGame::ghostFromId(int id)
{
	ghost_iterator G;

	G=m_Ghosts.find(id);
	if (G == m_Ghosts.end())
		return (NULL);
	else
		return ((*G).second);
}
//-----------------------------------------------------------------------
IGhostMaster *INetworkGame::masterFromId(int id)
{
	ghostmaster_iterator M;

	M=m_GhostMasters.find(id);
	if (M == m_GhostMasters.end())
		return (NULL);
	else
		return ((*M).second);
}
//-----------------------------------------------------------------------
void INetworkGame::onClientDisconnection(int cli)
{
	map<int,IGhost *> tmp;
	ghost_iterator    G=m_Ghosts.begin();

	while (m_Ghosts.end() != G)
	{
		if ((*G).second->getMasterClientId() == cli)
			(*G).second->onGhostMasterDied();
		else
			tmp[(*G).first]=(*G).second;
		G++;
	}
	m_Ghosts=tmp;
}
//-----------------------------------------------------------------------
void INetworkGame::onClientConnection(int cli)
{

}
// ---------------------------------------------------------- 
// Sylvain Lefebvre - (c) 2002 - Sylvain.Lefebvre@laposte.net
// ----------------------------------------------------------
