/**
   \class IPersistObject

   \brief Interface for persistents objects

   Objects ids should *not* be modified by a class
   that is not CPersistFactory !
*/
// --------------------------------------------------------
//  Author : Sylvain Lefebvre
//  History: Created ... 28/12/2001
// --------------------------------------------------------
#ifndef __PERSISTENT_OBJECT__
#define __PERSISTENT_OBJECT__
// --------------------------------------------------------
#include <iostream>
#include <string>
// --------------------------------------------------------
#include "CException.h"
#include "CPersistFactory.h"
#include "IPersistType.h"
#include "MAllocType.h"
// --------------------------------------------------------
class IPersistObject : public MAllocType
{
 private:

  int                     m_iId;
  int                     m_iIdStack; // for now, no need of a real stack
  
  void setId(int i)       {m_iId=i;}
  int  getId() const      {if (m_iId < 0) throw CException("IPersistObject: trying to access undefined id !"); return (m_iId);}
  int  idDefined() const  {return (m_iId >= 0);}
  void pushId() {m_iIdStack=m_iId; m_iId=-1;}
  void popId()  {m_iId=m_iIdStack; m_iIdStack=-1;}

  friend class CPersistFactory;

 protected:

  CPersistFactory     *m_Factory;

  //! called by <i>loading constructor</i> before loading
  virtual void init()         {};
  //! called by <i>loading constructor</i>
  virtual void load(istream& ){}; 
  //! called after load complete
  virtual void finalize()     {};
  //! load failed - init with defaults
  virtual void failed()       {};
  //! called when object need to be saved
  virtual void save(ostream& ){};
  //! called to load/save members - DO NOT IMPLEMENT -
  virtual void rawPersistLoadSaveMembers(istream *,ostream *) {}
#ifdef PERSIST_XML
  //! called to load/save members in xml format - DO NOT IMPLEMENT -
  virtual list<CPersistFactory::CPtrStock *> xmlPersistLoadSaveMembers(const char *,
																	   vector<string>&,
																	   ostream *)=0;
# endif

 public:

           IPersistObject(istream& );
           IPersistObject();
           IPersistObject(bool dynamic);

  virtual ~IPersistObject();

  virtual IPersistType   *getPersistType() 
    {
      throw CException("IPersistObject: Object has no type id !!\n\tPlease use PERSIST_ID macro to create a type id for your objects."); 
      return (NULL);
    }
  
};
// --------------------------------------------------------
#endif
// --------------------------------------------------------
