// ----------------------------------------------------
/*
   class CInterface

   Author :   Sylvain Lefebvre
   Created:   06/11/2002

 */
// ----------------------------------------------------
#ifndef __CINTERFACE_H_
#define __CINTERFACE_H_
// ----------------------------------------------------
#include "interface.h"
// ----------------------------------------------------
#include <vector>
#include <string>
#include <map>
// ----------------------------------------------------
class CMobile;
class QPushButton;
#ifdef QT3_SUPPORT
	#include <qhboxlayout>
	typedef QHBoxLayout QHBox;
#else
	class QHBox;
#endif
// ----------------------------------------------------
typedef struct s_color
{
public:

  int   m_iClr;
  const char *m_szName;

  s_color(int c,const char *n) : m_iClr(c), m_szName(n) {}

}color;
// ----------------------------------------------------
class CInterface : public Interface
{
  Q_OBJECT

 private:

  CMobile *m_Selected;

  typedef std::vector<QString> t_presets;

  t_presets m_sMvtPresets;
  t_presets m_sCollidePresets;
  t_presets m_sBordRPresets, m_sBordLPresets, m_sBordTPresets, m_sBordBPresets;
  t_presets m_sStartPresets;
  
  static std::vector<struct s_color>  s_Colors;

  void initComboIcons();
  void initComboColors();

  std::map<int,QPushButton *>         m_Buttons;
  bool                                m_bSave;
  bool                                m_bPresets;
  bool                                m_bHotkeys;
  QHBox                              *m_hbVisu;
  QString                             m_DirSessions;
  QString                             m_DirMobiles;

  QString extractPath(const QString& str);

 public:

	 CInterface(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = 0, char*presetsDir=0, char*mobilesDir=0);

  void updateDisplay(bool update_exprs=true);
  void refreshDisplay(bool force=false);
  void activeSave(bool v);
  void toggleSave();
  void activePresets(bool v);
  void togglePresets();
  void activeHotkeys(bool v);
  void toggleHotkeys();
  bool hotkeys() { return m_bHotkeys; }

  virtual void showEvent(QShowEvent *);
  virtual void wheelEvent(QWheelEvent *);

  CMobile *selected() {return (m_Selected);}
  bool     isClientVisible(int cli);
  void     setExprsPaperColor();

  public slots:

  virtual void addMobile();
  virtual void activateMobile(int n);
  virtual void applyExprs();
  virtual void applyAll();
  virtual void cancelExprs();
  virtual void exprsChanged();
  virtual void changeColor(int);
  virtual void startStopSelected();
  virtual void setValues();
  virtual void supprSelected();
  virtual void cloneSelected();
  virtual void load();
  virtual void saveSelected();
  virtual void exportChanged();
  virtual void visibleChanged();
  virtual void loadSession();
  virtual void loadSession(QString &s);
  virtual void saveSession();
  virtual void iconChange(int n);
  virtual void initPreset(t_presets &, QComboBox *, const char *);
  virtual void loadMvtPreset(int);
  virtual void loadCollidePreset(int);
  virtual void loadBordLPreset(int);
  virtual void loadBordRPreset(int);
  virtual void loadBordTPreset(int);
  virtual void loadBordBPreset(int);
  virtual void loadStartPreset(int);
  virtual void resetSelected();
  virtual void resetSession();
  virtual void viewAll();
  virtual void viewNone();
  virtual void exportAll();
  virtual void exportNone();
  virtual void selectPrev();
  virtual void selectNext();
  virtual void addVisuButton(int cli,bool enabled=true);
  virtual void removeVisuButton(int cli);
  
};
// ----------------------------------------------------
extern CInterface *g_MainWindow;
// ----------------------------------------------------
#endif
// ----------------------------------------------------
