// ----------------------------------------------------
/*
   class CMobile

   Author :   Sylvain Lefebvre
   Created:   06/11/2002

 */
// ----------------------------------------------------
#ifndef __CMOBILE_H_
#define __CMOBILE_H_
// ----------------------------------------------------
#include <string>
#include "CVertex.h"
#include "CIconFactory.h"
// ----------------------------------------------------
#include "IGhost.h"
#include "IGhostMaster.h"
// ----------------------------------------------------
#define PERSIST_XML
#include <persist.h>
// ----------------------------------------------------
#include <persist_string.h>
// ----------------------------------------------------
#define MOBILE_STATE_STOPPED         1

#define MOBILE_MSG_EXPORT_TRUE       0
#define MOBILE_MSG_EXPORT_FALSE      1
#define MOBILE_MSG_VISIBLE_TRUE      2
#define MOBILE_MSG_VISIBLE_FALSE     3
#define MOBILE_MSG_EXEC_STOP         4
#define MOBILE_MSG_EXEC_START        5
#define MOBILE_MSG_EXEC_MOVETO       6
#define MOBILE_MSG_EXEC_RESTART      7

#define FIELD_MVT     1
#define FIELD_START   2
#define FIELD_COLLIDE 3
#define FIELD_BORDER0 4
#define FIELD_BORDER1 5
#define FIELD_BORDER2 6
#define FIELD_BORDER3 7
// ----------------------------------------------------
typedef struct
{
  float x;
  float y;
}mobile_moveto_msg;
// ----------------------------------------------------
#define MAX_VAR                      4
// ----------------------------------------------------
class CalcParser;
// ----------------------------------------------------
typedef struct mobile_data
{
  float         pos[2];
  float         speed[2]; // displacement, in facts (i.e. dt included)
  float angle;
  unsigned int  color;
  unsigned int  icon;
  float         h;
  float         w;
}mobile_data;
// ----------------------------------------------------
class CMobile : public IGhost, public IGhostMaster, public IPersistObject
{
private:

  CVertex       m_Pos;
  CVertex       m_Speed; // displacement, in facts (i.e. dt included)
  double        m_dAngle;
  unsigned int  m_uiColor;
  int           m_iLastTime;
  int           m_iUpdateRate;
  double        m_dHeight;
  double        m_dWidth;
  int           m_iExprLastError;
  int           m_iExprLineError;
  bool          m_bEditing;
  int           m_iRank;

  double        m_dLength;
  double        m_dRadius;

  double        m_Vars[MAX_VAR+1];

  bool          m_bExport;
  bool          m_bVisible;
  bool          m_bValid;
  bool          m_bContact;
  bool          m_bContactFirst;
  int           m_uiState;
  int           m_iIcon;
  bool          m_bStarting;

  persist_string m_sExprs;
  persist_string m_sExprsCollide;
  persist_string m_sExprsBorder[4];
  persist_string m_sExprsStart;
  CalcParser   *m_ParserExprs;
  CalcParser   *m_ParserExprsCollide;
  CalcParser   *m_ParserExprsBorder[4];
  CalcParser   *m_ParserExprsStart;

public:
  
  CMobile();
  CMobile(int t,int id,const unsigned char *d,int l);
  CMobile(CMobile *m);
  PERSIST_OBJECT(CMobile)

  virtual ~CMobile();

  virtual void init();
  virtual void finalize() {compile();}

  virtual void draw();
  void         compile();
  void         update();
  void         reset();
  void         start();
  void         collide(CMobile *c,const CVertex& dir);
  void         collideBorder(int);
  void         startParserForField(void* walker, int i);
  void         updateExport();
  void         updateVisible();

  const CVertex& getPos() const {return (m_Pos);}
  void           setPos(const CVertex& p) {m_Pos=p;}
  CVertex&       pos() {return (m_Pos);}

  const CVertex& getSpeed() const {return (m_Speed);}
  void           setSpeed(const CVertex& p) {m_Speed=p;}
  CVertex&       speed() {return (m_Speed);}

  double         getAngle() const {return (m_dAngle);}
  void           setAngle(double a) {m_dAngle=a;}
  
  unsigned int   getColor() const {return (m_uiColor);}
  void           setColor(unsigned int c) {m_uiColor=c;}

  double         getWidth() const {return (m_dWidth);}
  void           setWidth(double l) {m_dWidth=l;}

  double         getHeight() const {return (m_dHeight);}
  void           setHeight(double l) {m_dHeight=l;}

  int            getIcon() const {return (m_iIcon);}
  void           setIcon(int a) {m_iIcon=(a % CIconFactory::getNbIcons());}

  double         getLength() const {return (m_dLength);}
  void           setLength(double l) {m_dLength=l;}

  double         getRadius() const {return (m_dRadius);}
  void           setRadius(double l) {m_dRadius=l;}

  double         getVar(int i) const { if (0<=i && i<=MAX_VAR) return (m_Vars[i]); else return 0.0; }
  void           setVar(int i, double a) { if (0<=i && i<=MAX_VAR) m_Vars[i]=a;}

  std::string    getExprs() const {return (m_sExprs.str());}
  void           setExprs(std::string s) {m_sExprs.str()=s;}

  std::string    getExprsCollide() const {return (m_sExprsCollide.str());}
  void           setExprsCollide(std::string s) {m_sExprsCollide.str()=s;}

  std::string    getExprsBorder(int b) const {return (m_sExprsBorder[b].str());}
  void           setExprsBorder(int b,std::string s) {m_sExprsBorder[b].str()=s;}

  std::string    getExprsStart() const {return (m_sExprsStart.str());}
  void           setExprsStart(std::string s) {m_sExprsStart.str()=s;}

  bool           isValid() const {return (m_bValid);}

  int            getExprLastError() const {return (m_iExprLastError);}
  int            getExprLineError() const {return (m_iExprLineError);}

  bool           isExported() const {return (m_bExport);}
  void           setExport(bool b){m_bExport=b;}

  bool           isVisible() const {return (m_bVisible);}
  void           setVisible(bool b){m_bVisible=b;}
  
  bool           isStopped() const {return (m_uiState & MOBILE_STATE_STOPPED);}
  void           setStopped(bool b);

  bool           firstContact() const {return (m_bContactFirst);}
  void           setContact(bool c) {m_bContact=c;}

  bool           isEdited() const {return (m_bEditing);}
  void           setEditing(bool b);

  int            getRank() {return (m_iRank);}
  void           setRank(int r) {m_iRank=r;}

  void           networkUpdate();

  // script events
  void           execStop();
  void           execStart();
  void           execRestart();
  void           execMoveTo(double x,double y);

  // methods from INetworkObject

  virtual       int            networkDataSize();
  virtual const unsigned char *writeNetworkData();
  virtual       void           readNetworkData(const unsigned char *d,int l);

  // methods from IGhost
  virtual       void           onGhostMasterDied();
  virtual       void           messageFromMaster(ng_ghost_msg *,const unsigned char *,int);

  // methods from IGhostMaster
  virtual       void           messageFromGhost(int,const ng_ghost_msg *,
						const unsigned char *,int);

  // comparator
  class less
  {
  public:
    bool operator () (CMobile *m0,CMobile *m1)
      {
	if (m0 == NULL)
	  return (false);
	if (m1 == NULL)
	  return (true);
	return (m0->getGhostId() < m1->getGhostId());
      }
  };
};
// ----------------------------------------------------
#endif
// ----------------------------------------------------
