#ifdef WIN32
#  include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glut.h>
#include "CIcon.h"
#include <string>

#define ICON_SCALE 5.

typedef CIcon icones;

class stdmesh: public icones {
protected:
  GLuint list;     // id display list
  GLuint text;     // id texture bind
  int    type;     // flags.  e.g. texture with alpha
  GLuint initlist();
  GLuint inittext(); // utils
  void freelist();
  void freetext();
  // si display-list:
  //    constructeurs des feuilles = begininit() + trace' GL + endinit()
  // sinon:
  //    virtual draw = pushattrb + trace' + popattrb
  void begininit(const char* tex = 0, const char* alpha = 0);
  void endinit();
public:
  virtual void  draw(double sh=1., double sr=1.);
  stdmesh(const char* name=0, double r=ICON_SCALE,double h=ICON_SCALE, double a=0):icones(name,r,h,a)
         { list=0; text=0; type=0; };
  virtual ~stdmesh() { freelist(); freetext(); };
};

class image: public stdmesh {
public:
  image(const char* name, const char* icon = 0, const char* alpha = 0);
  // virtual void  draw(double sh=1., double sr=1.);
};

class imagepure: public stdmesh {
public:
  imagepure(const char* name, const char* icon = 0, const char* alpha = 0);
};

class planet: public stdmesh {
public:
  planet(const char* name, const char* icon = 0, const char* alpha = 0);
};

// --- classes particulieres ------

class sphere: public stdmesh {
public:
  sphere();
};

class balle: public stdmesh {
public:
  balle();
};

class rond: public stdmesh {
public:
  rond();
};

class spot: public stdmesh {
public:
  spot();
};

class cercle: public stdmesh {
public:
  cercle();
};

class volant: public stdmesh {
public:
  volant();
};

class cube: public stdmesh {
public:
  cube();
};

class brique: public stdmesh {
public:
  brique();
};

class but: public stdmesh {
public:
  but(double l=3., double h=3, double p=1.5, char * dataDir=0);
};

class teapot: public stdmesh {
public:
  teapot();
};

class paroie: public stdmesh {
public:
  paroie(double l=6.);
};

class paroie2: public stdmesh {
public:
  paroie2():stdmesh("paroie2",10.,1) {};
  void draw(double sh=1., double sr=1);
};

class laser: public stdmesh {
public:
  laser():stdmesh("laser",ICON_SCALE,.1*ICON_SCALE) {};
  void draw(double sh=1., double sr=1.);
};

class triangle: public stdmesh {
public:
  triangle():stdmesh("triangle",10.,10.) {};
  void draw(double sh=1., double sr=1.);
};

class trait: public stdmesh {
public:
  trait():stdmesh("trait",.5 /* 2*ICON_SCALE */ ,.1) {};
  void draw(double sh=1., double sr=1.);
};

class ressort: public stdmesh {
public:
  ressort():stdmesh("ressort",.5 /* 2*ICON_SCALE */ , 1.) {};
  void draw(double sh=1., double sr=1.);
};

class aiguille: public stdmesh {
public:
  aiguille():stdmesh("aiguille",2*ICON_SCALE,.1) {};
  void draw(double sh=1., double sr=1.);
};

class raquette: public stdmesh {
public:
  raquette(char * dataDir);
};

class soucoupe: public stdmesh {
public:
  soucoupe();
};

class fusee: public stdmesh {
public:
  fusee();
};

class avion: public stdmesh {
public:
  avion();
};

class fleche: public image {
public:
  fleche(char * dataDir):image("fleche",(std::string(dataDir)+std::string("/../data/fleche.ppm")).c_str() ,(std::string(dataDir)+std::string("/../data/fleche.mask.ppm")).c_str())
  { m_dHeight=.35*ICON_SCALE; m_dRadius=.3*ICON_SCALE; };
};

class cible: public image {
public:
  cible(char * dataDir):image("cible",(std::string(dataDir)+std::string("/../data/cible.ppm")).c_str() ,(std::string(dataDir)+std::string("/../data/cible.mask.ppm")).c_str() )
  { m_dHeight=.5*ICON_SCALE; m_dRadius=.5*ICON_SCALE; };
};

class main: public image {
public:
  main(char * dataDir):image("main",(std::string(dataDir)+std::string("/../data/main.ppm")).c_str() ,(std::string(dataDir)+std::string("/../data/main.ppm")).c_str())
  { m_dHeight=.9*ICON_SCALE; m_dRadius=.7*ICON_SCALE; };
};

class digit0: public image {
public:
  digit0(char * dataDir):image("zero",(std::string(dataDir)+std::string("/../data/digit0.ppm")).c_str()){};
};

class digit1: public image {
public:
  digit1(char * dataDir):image("un",(std::string(dataDir)+std::string("/../data/digit1.ppm")).c_str()){};
};

class digit2: public image {
public:
  digit2(char * dataDir):image("deux",(std::string(dataDir)+std::string("/../data/digit2.ppm")).c_str()){};
};

class digit3: public image {
public:
  digit3(char * dataDir):image("trois",(std::string(dataDir)+std::string("/../data/digit3.ppm")).c_str()){};
};

class digit4: public image {
public:
  digit4(char * dataDir):image("quatre",(std::string(dataDir)+std::string("/../data/digit4.ppm")).c_str()){};
};

class digit5: public image {
public:
  digit5(char * dataDir):image("cinq",(std::string(dataDir)+std::string("/../data/digit5.ppm")).c_str()){};
};

class digit6: public image {
public:
  digit6(char * dataDir):image("six",(std::string(dataDir)+std::string("/../data/digit6.ppm")).c_str()){};
};

class digit7: public image {
public:
  digit7(char * dataDir):image("sept",(std::string(dataDir)+std::string("/../data/digit7.ppm")).c_str()){};
};

class digit8: public image {
public:
  digit8(char * dataDir):image("huit",(std::string(dataDir)+std::string("/../data/digit8.ppm")).c_str()){};
};

class digit9: public image {
public:
  digit9(char * dataDir):image("neuf",(std::string(dataDir)+std::string("/../data/digit9.ppm")).c_str()){};
};

class asteroide:  public image {
public:
  asteroide(char * dataDir):
  image("asteroide",(std::string(dataDir)+std::string("/../data/asteroid2.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/asteroid2.mask.ppm")).c_str())
  { m_dHeight=.8*ICON_SCALE; m_dRadius=.4*ICON_SCALE; m_dAngle0=1.5708; };
};

class soleil:  public imagepure {
public:
  soleil(char * dataDir): imagepure("soleil",(std::string(dataDir)+std::string("/../data/sun.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/sun.mask.ppm")).c_str()){};
};

class mercure:  public planet {
public:
  mercure(char * dataDir): planet("mercure",(std::string(dataDir)+std::string("/../data/mercury.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/mercury.mask.ppm")).c_str()){};
};

class venus:  public planet {
public:
  venus(char * dataDir): planet("venus",(std::string(dataDir)+std::string("/../data/venus.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/venus.mask.ppm")).c_str()){};
};

class terre:  public planet {
public:
  terre(char * dataDir): planet("terre",(std::string(dataDir)+std::string("/../data/earth.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/earth.mask.ppm")).c_str()){};
};

class lune:  public planet {
public:
  lune(char * dataDir): planet("lune",(std::string(dataDir)+std::string("/../data/moon.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/moon.mask.ppm")).c_str()){};
};

class mars:  public imagepure {
public:
  mars(char * dataDir): imagepure("mars",(std::string(dataDir)+std::string("/../data/mars.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/mars.mask.ppm")).c_str()){};
};

class jupiter:  public imagepure {
public:
  jupiter(char * dataDir): imagepure("jupiter",(std::string(dataDir)+std::string("/../data/jupiter.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/jupiter.mask.ppm")).c_str()){};
};

class saturne:  public imagepure {
public:
  saturne(char * dataDir): imagepure("saturne",(std::string(dataDir)+std::string("/../data/saturn.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/saturn.mask.ppm")).c_str())
  { m_dHeight=.5*ICON_SCALE; m_dRadius=.5*ICON_SCALE; };
};

class uranus:  public imagepure {
public:
  uranus(char * dataDir): imagepure("uranus",(std::string(dataDir)+std::string("/../data/uranus.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/uranus.mask.ppm")).c_str()){};
};

class neptune:  public imagepure {
public:
  neptune(char * dataDir): imagepure("neptune",(std::string(dataDir)+std::string("/../data/neptune.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/neptune.mask.ppm")).c_str()){};
};

class pluton:  public imagepure {
public:
  pluton(char * dataDir):   imagepure("pluton",(std::string(dataDir)+std::string("/../data/pluton.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/pluton.mask.ppm")).c_str()){};
};

class etoile:  public image {
public:
  etoile(char * dataDir):   image("etoile",(std::string(dataDir)+std::string("/../data/star.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/star.ppm")).c_str()){};
};

class gagne:  public image {
public:
  gagne(char * dataDir):   image("gagne'",(std::string(dataDir)+std::string("/../data/gagne.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/gagne.ppm")).c_str()){};
};

class perdu:  public image {
public:
  perdu(char * dataDir):   image("perdu",(std::string(dataDir)+std::string("/../data/perdu.ppm")).c_str(),(std::string(dataDir)+std::string("/../data/perdu.ppm")).c_str()){};
};


